;Display Test for ECM-A0428 LCD
;ATtiny2313 @ 14.7456 MHz (external XTAL)
;
;PORTS
;  PA0-1 = XTAL
;  PB0-7 = LCD Data
;  PD0   = RS232 Rx
;  PD1   = RS232 Tx [unused]
;  PD2   = LCD #RESET
;  PD3   = LCD A0
;  PD4   = LCD #WR
;  PD5   = LCD #RD
;  PD6   = LCD #CS

.include "../include/tn2313def.inc"

.def	temp = R16
.def	temp2 = R17

.equ	LCD_RESET = 2
.equ	LCD_A0 = 3
.equ	LCD_WR = 4
.equ	LCD_RD = 5
.equ	LCD_CS = 6

.equ	TEXT_ADDR = 0x0000
.equ	GFX_ADDR = 0x0400
.equ	CHARWIDTH = 6
.equ	CHARS_PER_LINE = (280 + CHARWIDTH-1) / CHARWIDTH

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
	
;===============================================================================

text:
.if (CHARWIDTH == 6)

.db "Lorem ipsum dolor sit amet, consetetur         ", \
    "sadipscing elitr, sed diam nonumy eirmod tempor"
.db "invidunt ut labore et dolore magna aliquyam    ", \
    "erat, sed diam voluptua. At vero eos et accusam"
.db "et justo duo dolores et ea rebum. Stet clita   ", \
    "kasd gubergren, no sea takimata sanctus est    "
.db "Lorem ipsum dolor sit amet. Lorem ipsum dolor  ", \
    "sit amet, consetetur sadipscing elitr, sed diam"

.db "voluptua. At vero eos et accusam et justo duo  ", \
    "dolores et ea rebum. Stet clita kasd gubergren,"
.db "no sea takimata sanctus est Lorem ipsum dolor  ", \
    "sit amet. Lorem ipsum dolor sit amet,          "
.db "consetetur sadipscing elitr, sed diam nonumy   ", \
    "eirmod tempor invidunt ut labore et dolore     "
.db "magna aliquyam erat, sed diam voluptua. At vero", \
    "eos et accusam et justo duo dolores et ea rebum"

.elseif (CHARWIDTH == 8)

.db "Lorem ipsum dolor sit amet,        ", \
    "consetetur sadipscing elitr, sed   "
.db "diam nonumy eirmod tempor invidunt ", \
    "ut labore et dolore magna aliquyam "
.db "erat, sed diam voluptua. At vero   ", \
    "eos et accusam et justo duo dolores"
.db "et ea rebum. Stet clita kasd       ", \
    "gubergren, no sea takimata sanctus "
.db "est Lorem ipsum dolor sit amet.    ", \
    "Lorem ipsum dolor sit amet,        "
.db "consetetur sadipscing elitr, sed   ", \
    "diam nonumy eirmod tempor invidunt "

.endif

.db 0, 0

;===============================================================================

reset:
	ldi	temp, LOW(RAMEND)
	out	SPL, temp
	
	ldi	temp, 0xFF
	out	DDRB, temp
	ldi	temp, 0x00
	out	PORTB, temp
	ldi	temp, 0x7E
	out	DDRD, temp
	ldi	temp, 1<<LCD_WR | 1<<LCD_RD | 1<<LCD_RESET | 1<<LCD_CS | 0x02
	out	PORTD, temp
	
	ldi	temp, 1<<RXEN | 1<<TXEN
	out	UCSRB, temp
	ldi	temp, 7 ;115200 Baud @ 14.7456 MHz
	out	UBRRL, temp

	rcall	lcd_delay
	cbi	PORTD, LCD_RESET
	rcall	lcd_delay
	sbi	PORTD, LCD_RESET
	rcall	lcd_delay
	
napf:
	sbis	UCSRA, RXC
	rjmp	PC-1
	in	temp, UDR
	rcall	lcd_cmd
	rcall	lcd_delay
	rcall	lcd_read
	out	UDR, temp
	rjmp	napf


	rcall	lcd_init
	rcall	lcd_clear
	
	
	
	;text mode test
	
	ldi	temp, 0x46 ;CSRW
	rcall	lcd_cmd
	ldi	temp, LOW(TEXT_ADDR) ;set address
	rcall	lcd_data
	ldi	temp, HIGH(TEXT_ADDR)
	rcall	lcd_data
	
	ldi	temp, 0x42 ;Mwrite
	rcall	lcd_cmd
	
	ldi	ZL, LOW(text*2)
	ldi	ZH, HIGH(text*2)
lcd_test_loop:
	lpm	temp, Z+
	cpi	temp, 0
	breq	lcd_test_end
	rcall	lcd_data
	rjmp	lcd_test_loop
lcd_test_end:

	;graphics mode test
lcd_gfxtest_start:
	ldi	temp, 0x46 ;CSRW
	rcall	lcd_cmd
	ldi	temp, LOW(GFX_ADDR) ;set address
	rcall	lcd_data
	ldi	temp, HIGH(GFX_ADDR)
	rcall	lcd_data
	
	ldi	temp, 0x42 ;Mwrite
	rcall	lcd_cmd
	
	ldi	ZL, LOW(128*CHARS_PER_LINE)
	ldi	ZH, HIGH(128*CHARS_PER_LINE)
lcd_gfxtest_loop:
	sbis	UCSRA, RXC
	rjmp	lcd_gfxtest_loop
	in	temp, UDR
	rcall	lcd_data
	sbiw	ZH:ZL, 1
	brne	lcd_gfxtest_loop
	rjmp	lcd_gfxtest_start

;-------------------------------------------------------------------------------

loop:
	rjmp	loop

;===============================================================================

lcd_cmd:
	;send command to LCD
	out	PORTB, temp
	sbi	PORTD, LCD_A0
	cbi	PORTD, LCD_CS
	nop
	cbi	PORTD, LCD_WR
	rjmp	PC+1
	sbi	PORTD, LCD_WR
	sbi	PORTD, LCD_CS
	ret
	
;--------------------

lcd_data:
	;send data to LCD
	out	PORTB, temp
	cbi	PORTD, LCD_A0
	cbi	PORTD, LCD_CS
	nop
	cbi	PORTD, LCD_WR
	rjmp	PC+1
	sbi	PORTD, LCD_WR
	sbi	PORTD, LCD_CS
	ret

;--------------------

lcd_read:
	;read data from LCD
	ldi	temp, 0x00
	out	DDRB, temp
	out	PORTB, temp
	cbi	PORTD, LCD_A0
	cbi	PORTD, LCD_CS
	cbi	PORTD, LCD_RD
	rjmp	PC+1
	rjmp	PC+1
	in	temp, PINB
	sbi	PORTD, LCD_RD
	sbi	PORTD, LCD_CS
	ldi	temp2, 0xFF
	out	DDRB, temp
	ret

;--------------------

lcd_delay:
	;approx. 5ms delay @ 14.7456 MHz
	ldi	temp, 100
	clr	temp2
	dec	temp2
	brne	PC-1
	dec	temp
	brne	PC-3
	ret

;--------------------

lcd_clear:
	;clear text mode
	ldi	temp, 0x46 ;CSRW
	rcall	lcd_cmd
	ldi	temp, LOW(TEXT_ADDR) ;set address
	rcall	lcd_data
	ldi	temp, HIGH(TEXT_ADDR)
	rcall	lcd_data
	
	ldi	temp, 0x42 ;Mwrite
	rcall	lcd_cmd
	ldi	ZL, LOW(12*CHARS_PER_LINE)
	ldi	ZH, HIGH(12*CHARS_PER_LINE)
lcd_clear_loop1:
	ldi	temp, 0x00
	rcall	lcd_data
	sbiw	ZH:ZL, 1
	brne	lcd_clear_loop1
	
	;clear graphics mode
	ldi	temp, 0x46 ;CSRW
	rcall	lcd_cmd
	ldi	temp, LOW(GFX_ADDR) ;set address
	rcall	lcd_data
	ldi	temp, HIGH(GFX_ADDR)
	rcall	lcd_data
	
	ldi	temp, 0x42 ;Mwrite
	rcall	lcd_cmd
	ldi	ZL, LOW(128*CHARS_PER_LINE)
	ldi	ZH, HIGH(128*CHARS_PER_LINE)
lcd_clear_loop2:
	ldi	temp, 0x00
	rcall	lcd_data
	sbiw	ZH:ZL, 1
	brne	lcd_clear_loop2
	
	ret

;--------------------

lcd_init:
	;reset and initialize LCD
	rcall	lcd_delay
	cbi	PORTD, LCD_RESET
	rcall	lcd_delay
	sbi	PORTD, LCD_RESET
	rcall	lcd_delay
	
	ldi	temp, 0x40 ;System Set
	rcall	lcd_cmd
	ldi	temp, 0x38 ;int. CGROM, 8px high chars, dual panel, LCD mode
	rcall	lcd_data
	ldi	temp, 0x80 | CHARWIDTH-1 ;2-frame AC drive, 8px char width
	rcall	lcd_data
	ldi	temp, 11-1 ;char height: 11 lines
	rcall	lcd_data
	ldi	temp, CHARS_PER_LINE-1 ;chars per line
	rcall	lcd_data
.if (CHARWIDTH == 8)
	ldi	temp, 41 ;line length including blanking (value for 8px chars)
.elseif (CHARWIDTH == 6)
	ldi	temp, 55 ;line length including blanking (value for 6px chars)
.else
	.error "Character width not supported (use 6 or 8 pixel wide chars)"
.endif
	rcall	lcd_data
	ldi	temp, 132-1 ;132 display lines (last 4 missing ...)
	rcall	lcd_data
	ldi	temp, LOW(CHARS_PER_LINE) ;horizontal address range
	rcall	lcd_data
	ldi	temp, HIGH(CHARS_PER_LINE)
	rcall	lcd_data
	
	ldi	temp, 0x44 ;Scroll
	rcall	lcd_cmd
	ldi	temp, LOW(TEXT_ADDR) ;character page 1 address
	rcall	lcd_data
	ldi	temp, HIGH(TEXT_ADDR)
	rcall	lcd_data
	ldi	temp, 66-1 ;66px for each panel (text)
	rcall	lcd_data
	ldi	temp, LOW(GFX_ADDR) ;graphics page 1 address
	rcall	lcd_data
	ldi	temp, HIGH(GFX_ADDR)
	rcall	lcd_data
	ldi	temp, 66-1 ;66px for each panel (graphics)
	rcall	lcd_data
	ldi	temp, LOW(TEXT_ADDR + 6*CHARS_PER_LINE) ;character page 2 address
	rcall	lcd_data
	ldi	temp, HIGH(TEXT_ADDR + 6*CHARS_PER_LINE)
	rcall	lcd_data
	ldi	temp, LOW(GFX_ADDR + 66*CHARS_PER_LINE) ;graphics page 2 address
	rcall	lcd_data
	ldi	temp, HIGH(GFX_ADDR + 66*CHARS_PER_LINE)
	rcall	lcd_data
	
	ldi	temp, 0x5A ;Hdot Scr
	rcall	lcd_cmd
	ldi	temp, 0x00 ;no horizontal offset
	rcall	lcd_data
	
	ldi	temp, 0x5B ;Ovlay
	rcall	lcd_cmd
	ldi	temp, 0x00 ;two-layer comp, blocks 1 and 3 = text, OR mode
	rcall	lcd_data
	
	ldi	temp, 0x58 ;Disp off
	rcall	lcd_cmd
;	ldi	temp, 0x44 ;enable blocks 1 and 3 (static on), no cursor
	ldi	temp, 0x54 ;enable blocks 1, 3 and 2+4, no cursor
;	ldi temp, 0x10 ;GFX only TODO
	rcall	lcd_data
	
	ldi	temp, 0x5D ;CSRFORM
	rcall	lcd_cmd
	ldi	temp, CHARWIDTH-1 ;6 pixels wide
	rcall	lcd_data
	ldi	temp, 0x80 | 8-1 ;8 pixels high, block cursor
	rcall	lcd_data
	
	ldi	temp, 0x4C ;CSRDIR = right
	rcall	lcd_cmd
	
	ldi	temp, 0x59 ;Disp on
	rcall	lcd_cmd
;	ldi	temp, 0x44 ;enable blocks 1 and 3 (static on), no cursor
;	rcall	lcd_data
	
; ldi byte_to_send,0x5C ; Send Set CGRAM Address Command
; rcall SEND_COMMAND
; ldi byte_to_send,0x00 ; = low byte of CGRAM address
; rcall SEND_DATA
; ldi byte_to_send,0x04 ; = high byte of CGRAM address
; rcall SEND_DATA	

	ret
	
	