#!/usr/bin/perl -w
use Image::Magick;

# convert png image to binary
# horizontal bytes, MSB right
# creates two files with 6px/byte (left-adjusted) and 8px/byte

if (! @ARGV) {
	die ("Usage: ./gfx2asm.pl <Filename.png> [<Filename.png> [...]]\n");
}

foreach $file (@ARGV) {
	if (! -f $file) {
		die ("Image file '$file' not found!\n");
	}
	$image = Image::Magick->new;
	$image->Read($file);
	$width = $image->Get('width');
	$height = $image->Get('height');
	print "Converting '$file' ($width x $height) ...\n";
	@pixels = $image->GetPixels(map=>'I', width=>$width, height=>$height, normalize=>1);
	
	$filename = $file;
	$filename =~ s/\.[A-Za-z0-9]+$//;
#	open ASM, ">$filename.asm" or die ("Failed to open ASM file.\n");
	open ASM, ">/dev/null";
	open BIN6, ">6px_$filename.bin" or die ("Failed to open ASM file.\n");
	open BIN8, ">8px_$filename.bin" or die ("Failed to open ASM file.\n");
	#$bytewidth = int(($width+7)/8);
	
#	print ASM ".dw $width, $height"; # missing \n is intentional
	$n = 0;
	for ($y = 0; $y < $height; $y++) {
		for ($x = 0; $x < $width; $x+=8) {
			if ($n % 8 == 0) {
				print ASM "\n.db ";
			} else {
				print ASM ", ";
			}
			$byte = 0;
			for ($b = 0; ($b < 8) && ($x+$b < $width); $b++) {
				if ($pixels[$y*$width + $x+$b] < .5) {
					$byte |= 1 << (7-$b);
				}
			}
			printf ASM "0x%02x", $byte;
			print BIN8 pack('C', $byte);
			$n++;
		}
		for ($x = 0; $x < $width; $x+=6) {
			$byte = 0;
			for ($b = 0; ($b < 6) && ($x+$b < $width); $b++) {
				if ($pixels[$y*$width + $x+$b] < .5) {
					$byte |= 1 << (7-$b);
				}
			}
			print BIN6 pack('C', $byte);
		}
	}
	print ASM "\n";
	close ASM;
	close BIN6;
	close BIN8;
}

